<?php
/*
 * Plugin Name:     Bonus for Woo
 * Version:         3.3.0
 * Text Domain:     bonus-for-woo
 * Plugin URI:      https://computy.ru/blog/bonus-for-woo-wordpress
 * Description:     This plugin adds a cashback system to the woocommerce functionality in the form of bonuses. Also, adding a percentage of cashback depending on the user's role. Also, changing the user's role, depending on the amount of all his purchases.
 * Author:          computy
 * Author URI:      https://computy.ru
 *
 * WC requires at least: 4.0
 * WC tested up to: 6.5.1
 *
 * License:           GNU General Public License v3.0
 * License URI:       https://www.gnu.org/licenses/gpl-3.0.html
 */

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

define('BONUS_COMPUTY_VERSION', '3.3.0');
define('BONUS_COMPUTY_PLUGIN_DIR', plugin_dir_path(__FILE__));
define('BONUS_COMPUTY_PLUGIN_URL',  plugin_dir_url( __FILE__ ) );



/*------------действия после обновления---------------------*/
add_action( 'upgrader_process_complete', 'bfw_update_completed', 10, 2 );
function bfw_update_completed( $upgrader_object, $options ) {
    $our_plugin = plugin_basename( __FILE__ );
    if( $options['action'] === 'update' && $options['type'] === 'plugin' && isset( $options['plugins'] ) ) {
        foreach( $options['plugins'] as $plugin ) {
            if( $plugin == $our_plugin ) {
                set_transient( 'bfw_pro_updated', 1 );
            }
        }
    }
}
require_once(BONUS_COMPUTY_PLUGIN_DIR . '/index/if-pro.php');
/*------------действия после обновления---------------------*/


/*проверка активен ли woocommerce*/
if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
} else {
    // деактивируем
    add_action('admin_init', 'bonus_for_woo_true_plugin_off');
    function bonus_for_woo_true_plugin_off()
    {
        deactivate_plugins(plugin_basename(__FILE__));
    }

    add_action('admin_notices', 'bonus_for_woo_true_plugin_uvedomlenie');
    function bonus_for_woo_true_plugin_uvedomlenie()
    {
        echo '<div id="message" class="notice notice-warning is-dismissible">
	<p>' . __('Woocommers is not activated. The <b> Bonus for Woo </b> plugin cannot be activated.', 'bonus-for-woo') . '</p>
</div>';
        if (isset($_GET['activate'])) {
            unset($_GET['activate']);
        }
    }
}
/*проверка активирован ли woocommerce*/

add_filter( 'plugin_action_links', function($links, $file){
    //проверка - наш это плагин или нет
    if ( $file != plugin_basename(__FILE__) ){return $links;}
    // создаем ссылку
    $settings_link = sprintf('<a href="%s">%s</a>', admin_url('admin.php?page=bonus_for_woo-plugin-options'), __( 'Settings', 'bonus-for-woo' ));
    array_unshift( $links, $settings_link );
    return $links;}, 10, 2 );

/*Функция, которая запускается при активации плагина*/
register_activation_hook(__FILE__, 'bfwoo_computy_activate');
function bfwoo_computy_activate()
{
    delete_option('rewrite_rules');

    global $wpdb;

    $table_name = $wpdb->prefix . 'bfw_computy';
    if ($wpdb->get_var("SHOW TABLES LIKE '$table_name'") != $table_name) {
        $charset_collate = $wpdb->get_charset_collate();
        $sql = "CREATE TABLE $table_name (
		id mediumint(9) NOT NULL AUTO_INCREMENT,
		name varchar(50) DEFAULT '' NOT NULL,
		slug varchar(50) DEFAULT '' NOT NULL,
		percent varchar(50) DEFAULT '' NOT NULL,
		summa_start varchar(50) DEFAULT '' NOT NULL,
		PRIMARY KEY  (id)
	) $charset_collate;";

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);
    }

    $table_history_name = $wpdb->prefix . 'bfw_history_computy';
    if ($wpdb->get_var("SHOW TABLES LIKE '$table_history_name'") != $table_history_name) {
        $charset_collate = $wpdb->get_charset_collate();
        $sql = "CREATE TABLE $table_history_name (
		id mediumint(9) NOT NULL AUTO_INCREMENT,
		user INT(10) NOT NULL,
		date datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
		symbol VARCHAR(10) NOT NULL,
		points INT(10)  NOT NULL,
		orderz INT(10)  NOT NULL,
		comment_admin VARCHAR(100)  DEFAULT '' NOT NULL,
		status VARCHAR(10)  DEFAULT '' NOT NULL,
		PRIMARY KEY (id)
	) $charset_collate;";

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);
    }

    $table_time_points_name = $wpdb->prefix . 'bfw_time_points_computy';
    if ($wpdb->get_var("SHOW TABLES LIKE '$table_time_points_name'") != $table_time_points_name) {
        $charset_collate = $wpdb->get_charset_collate();
        $sql = "CREATE TABLE $table_time_points_name (
		id mediumint(9) NOT NULL AUTO_INCREMENT,
		user INT(10) NOT NULL,
		created_date datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
		off_date datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
		points INT(10) NOT NULL,
		status VARCHAR(10) DEFAULT '' NOT NULL,
		PRIMARY KEY (id)
	) $charset_collate;";

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);
    }
}


/*Функция, которая запускается при деактивации плагина*/
register_deactivation_hook(__FILE__, 'bfwoo_computy_deactivate');
function bfwoo_computy_deactivate()
{
    delete_option('rewrite_rules');

}

/*------------добавляем стили на самом сайте------------*/
add_action('wp_enqueue_scripts', 'bfwoo_computy_styles');

function bfwoo_computy_styles()
{
    wp_register_style('bonus-computy-style', plugin_dir_url(__FILE__) . '_inc/bonus-computy-style.css',array(),BONUS_COMPUTY_VERSION);
    wp_enqueue_style('bonus-computy-style');
}
/*------------добавляем стили на самом сайте------------*/


/*------------добавляем скрипты на самом сайте------------*/
add_action('wp_enqueue_scripts', 'bfwoo_computy_script');

function bfwoo_computy_script()
{
    wp_register_script('bonus-computy-script', plugin_dir_url(__FILE__) . '_inc/bonus-computy-script.js',array('jquery'), BONUS_COMPUTY_VERSION, true);
    wp_enqueue_script('bonus-computy-script');
}
/*------------добавляем скрипты на самом сайте------------*/

/*-------инициализация бонусных баллов-------*/
require_once(BONUS_COMPUTY_PLUGIN_DIR . '/classes/Points.php');
/*-------инициализация бонусных баллов-------*/

/*-------инициализация ролей-------*/
require_once(BONUS_COMPUTY_PLUGIN_DIR . '/classes/Roles.php');
/*-------инициализация ролей-------*/

/*-------email уведомления-------*/
require_once(BONUS_COMPUTY_PLUGIN_DIR . '/classes/Emails.php');
/*-------email уведомления-------*/


/*-------Реферальная система-------*/
 require_once(BONUS_COMPUTY_PLUGIN_DIR . '/index/referal.php');
/*-------Реферальная система-------*/


/*------------добавляем страницу бонусов в аккаунте клиента---------------------*/
require_once(BONUS_COMPUTY_PLUGIN_DIR . '/index/account.php');
/*------------добавляем страницу бонусов в аккаунте клиента---------------------*/


/*------------Страница админки*------------*/
if (is_admin() || (defined('WP_CLI') && WP_CLI)) {
    require_once(BONUS_COMPUTY_PLUGIN_DIR . 'classes/class.bonus-for-woo-admin.php');
    add_action('init', array('BfwAdmin', 'init'));
}


/*------------Страница админки------------*/


/*------------Действие когда статус заказа выполнен------------*/
require_once(BONUS_COMPUTY_PLUGIN_DIR . '/index/status_completed.php');
/*------------Действие когда статус заказ выполнен------------*/

/*------------Класс истории начислений------------*/
require_once(BONUS_COMPUTY_PLUGIN_DIR . '/classes/History.php');
/*------------Класс истории начислений------------*/

/*------------функционал временных баллов------------*/
require_once(BONUS_COMPUTY_PLUGIN_DIR . '/classes/TimePoints.php');
/*------------функционал временных баллов------------*/


/*------------Действие когда клиент подтверждает заказ------------*/
add_action('woocommerce_checkout_order_processed', 'new_order_bfw', 20, 1);
function new_order_bfw($order_id)
{
    $order = wc_get_order($order_id);//вызов инфы о  заказе
    $customer_user = $order->get_customer_id();//узнаем id покупателя

    //узнаем баллы клиента
    $computy_point_old = (new BfwPoints)->getPoints($customer_user);
    //узнаем баллы которые он решил списать
    $computy_point_fast = (new BfwPoints)->getFastPoints($customer_user);
    if ($computy_point_fast > 0) {
        $cout_point = $computy_point_old - $computy_point_fast;
        (new BfwPoints)->updatePoints($customer_user,$cout_point);//удаляем баллы со счета клиента
    }
    (new BfwPoints)->updateFastPoints($customer_user,0);//очищаем добавленную скидку



    if ($computy_point_fast > 0) {
    /*запись в историю*/
        $prichina = sprintf(__('Use of %s', 'bonus-for-woo'), (new BfwPoints())->pointsLabel(5));
        (new BfwHistory)->add_history($customer_user,'-',$computy_point_fast,$order_id,$prichina);
        /*запись в историю*/
    /*email*/
        $val = get_option( 'bonus_option_name' );
        $title_email = $val['email-when-order-confirm-title'] ?? __('Writing off bonus points', 'bonus-for-woo');


        /*шаблонизатор письма*/
        $text_email = $val['email-when-order-confirm-text'] ?? '';
        $user = get_userdata($customer_user);
        $get_referral        = get_user_meta($customer_user, 'bfw_points_referral', true );
        $text_email_array = array(
            '[user]'=>$user->display_name,
            '[order]'=>$order_id,
            '[points]'=>$computy_point_fast,
            '[total]'=>$cout_point,
            '[cause]'=>$prichina,
            '[referral-link]'=> esc_url(site_url().'?bfwkey='. $get_referral));
        $message_email = (new BfwEmail)->template($text_email,$text_email_array);
        /*шаблонизатор письма*/


        if(isset($val['email-when-order-confirm'])){
            if($val['email-when-order-confirm']=='1') {
                (new BfwEmail)->getMail($customer_user, '', $title_email, $message_email);
            }}
        /*email*/
    }
    //списание временных баллов
}
/*------------Действие когда клиент подтверждает заказ------------*/


/*сортировка массива По возрастанию*/
function array_multisort_value()
{
    $args = func_get_args();
    $data = array_shift($args);
    foreach ($args as $n => $field) {
        if (is_string($field)) {
            $tmp = array();
            foreach ($data as $key => $row) {
                $tmp[$key] = $row[$field];
            }
            $args[$n] = $tmp;
        }
    }
    $args[] = &$data;
    call_user_func_array('array_multisort', $args);
    return array_pop($args);
}



/*------------добавление бонусов когда добавлен отзыв о товаре------------*/
require_once(BONUS_COMPUTY_PLUGIN_DIR . '/index/review.php');
/*------------добавление бонусов когда добавлен отзыв о товаре------------*/


/*вывод текста над формой регистрации*/
add_action( 'woocommerce_register_form_start', 'bfw_form_register', 10, 1 );
function bfw_form_register(){
    $val = get_option('bonus_option_name');

    if(!empty($val['points-for-registration'])){
        echo '<div class="bfw-in-register-form">'.sprintf(__('Register and get %s %s.', 'bonus-for-woo'), $val['points-for-registration'], (new BfwPoints())->pointsLabel(5)).'</div>';
    }

}
/*вывод текста над формой регистрации*/


/* действия  при регистрации пользователя*/
add_action('user_register', 'action_points_for_regisration_bfw');
/**
 * @param int $user_id
 */
function action_points_for_regisration_bfw($user_id){


    $pro = get_option('bonus-for-woo-pro');
    if ($pro == 'active') {


        $val = get_option('bonus_option_name');
        $pointsforeg = $val['points-for-registration'];

        $mojno = 1;
        /*только для рефералов */

            if (!empty($val['register-points-only-referal'])) {
                /*только для рефералов*/
                $cookie_val = isset($_COOKIE['bfw_ref_cookie_set']) ? sanitize_text_field(wp_unslash($_COOKIE['bfw_ref_cookie_set'])) : '';
                if (!empty($cookie_val)) {
                    $mojno = 1;
                }else{
                    $mojno = 0;
                }
            }

/*начисление баллов*/
        if($mojno==1){
            if($pointsforeg>0){
                (new BfwPoints)->updatePoints($user_id,$pointsforeg);

                $prichina  = sprintf(__('%s for registration','bonus-for-woo'), (new BfwPoints())->pointsLabel(5));
                /*Записываем в историю*/
                (new BfwHistory)->add_history($user_id,'+',$pointsforeg,'0',$prichina);
                /*Записываем в историю*/

                /*отправляем email клиенту*/
                $user = get_userdata($user_id);
                /*шаблон письма*/
                $val = get_option('bonus_option_name');

                    $text_email = $val['email-when-register-text'] ?? '';


                $title_email = $val['email-when-register-title'] ?? __('Bonus points have been added to you!', 'bonus-for-woo');
                $get_referral = get_user_meta( $user_id, 'bfw_points_referral', true );
                $text_email_array = array('[referral-link]'=> esc_url(site_url().'?bfwkey='. $get_referral),'[user]'=>$user->display_name,'[points]'=>$pointsforeg,'[total]'=>$pointsforeg,'[cause]'=>$prichina);
                $message_email = (new BfwEmail)->template($text_email,$text_email_array);
                /*шаблон письма*/


                   if(!empty($val['email-when-register'])){
                        (new BfwEmail)->getMail($user_id, '', $title_email, $message_email);
                    }
                /*отправляем email клиенту*/
            }
        }


    }
}
/*начисление баллов при регистрации пользователя*/


/*
*  вывод Списания баллов в корзине  и в оформлении товаров
* Обрабатываем отправленные данные, когда клиент хочет потратить бонусы
*/

/*Вывод кэшбэка в корзине и в формлении товара*/
require_once(BONUS_COMPUTY_PLUGIN_DIR . '/index/cashback_in_cart.php');
/*Вывод кэшбэка в корзине и в формлении товара*/


/*Списание баллов в корзине и оформлении заказа*/
require_once(BONUS_COMPUTY_PLUGIN_DIR . '/index/spisanie_cart_checkout.php');
/*Списание баллов в корзине и оформлении заказа*/


/*трата баллов*/
 add_action('wp_ajax_computy_trata_points', 'bfwoo_trata_points');
 add_action('wp_ajax_nopriv_computy_trata_points', 'bfwoo_trata_points');

function bfwoo_trata_points()
{
    $balls = intval(sanitize_text_field($_POST['computy_input_points']));
    $userid = get_current_user_id();

    $allpoint = (new BfwPoints)->getPoints($userid);
    if ($allpoint < $balls) {
        echo __('Nice try, cheater.','bonus-for-woo');
    } else {
        (new BfwPoints)->updateFastPoints($userid,$balls);//выносим баллы на скидку
        wp_send_json_success($_POST['redirect']);
    }
}
/*трата баллов*/


/*Действие при нажатии кнопки экспорта баллов*/
if (isset($_GET['export_bfw_points'])) {
    $buffer = fopen(BONUS_COMPUTY_PLUGIN_DIR . '/file.csv', 'w');
    fputs($buffer, chr(0xEF) . chr(0xBB) . chr(0xBF));
    global $wpdb;
    $users = $wpdb->get_results("SELECT * FROM $wpdb->users ORDER BY ID");

    $titlexport = array('id user','User name','Points' );
    fputcsv($buffer, (array)$titlexport, ',');

    $prods = array();
    foreach($users as $user) {
        $points = get_user_meta($user->ID, 'computy_point', true) ?? 0;
        $prods[] = array(
            'id'       => $user->ID,
            'name'     => $user->user_nicename,
            'points'    =>  $points,
        );
    }
    foreach($prods as $val) {
        fputcsv($buffer, (array)$val, ';');
    }
    fclose($buffer);
}
/*Действие при нажатии кнопки экспорта баллов*/

/*-------экспорт csv файла бонусов-------*/
add_action('wp_ajax_bfw_export_bonuses', 'bfw_export_bonuses');
add_action('wp_ajax_nopriv_bfw_export_bonuses', 'bfw_export_bonuses');
function bfw_export_bonuses()
{
    $array = stripslashes($_POST['response']);
    $resp = json_decode($array, JSON_UNESCAPED_SLASHES);
    $url_export_file = $resp['data']['url']; /*ссылка на загруженный файл экспорта*/
    if (($fp = fopen($url_export_file, "r")) !== false) {
        while (($data = fgetcsv($fp, 0, ",")) !== false) {
            $list[] = $data;
        }
        fclose($fp);
    }

    foreach ($list as $data) {
            $id = (int)$data[0];
            $username = $data[1];
            $point = (int)$data[2];
            if(get_user_meta($id, 'computy_fast_point', true)!=$point){
                update_user_meta($id, 'computy_point', $point);
                /*Обновляем баллы пользователям*/
            }
    }
echo 'good';
}
/*-------экспорт csv файла бонусов-------*/


/*-------добавляем скидку-------*/
add_action('woocommerce_cart_calculate_fees', 'bfwoo_add_fee', 25, 1);

function bfwoo_add_fee()
{
    $val = get_option('bonus_option_name');

    if ((new BfwPoints)->getFastPoints(get_current_user_id()) > 0) {
        $userid = get_current_user_id();
        $computy_point_old = (new BfwPoints)->getFastPoints($userid); //узнаем баллы которые он решил списать
        WC()->cart->add_fee($val['bonus-points-on-cart'], -$computy_point_old);
    }
}
/*-------добавляем скидку-------*/


/*--------- очистка схваченных баллов----------*/
add_action('wp_ajax_clear_fast_bonus', 'bfwoo_clean_fast_bonus');
add_action('wp_ajax_nopriv_clear_fast_bonus', 'bfwoo_clean_fast_bonus');
function bfwoo_clean_fast_bonus()
{
    $userid = get_current_user_id();
    (new BfwPoints)->updateFastPoints($userid,0);
    wp_send_json_success($_POST['redirect']);

}
/*--------- очистка схваченных баллов----------*/


/*------------шорткоды------------*/
require_once(BONUS_COMPUTY_PLUGIN_DIR . '/index/shortcodes.php');
/*------------шорткоды------------*/


/*---------текст на странице товара (сколько бонусов получите)--------*/
require_once(BONUS_COMPUTY_PLUGIN_DIR . '/index/single-product-page.php');
/*---------текст на странице товара (сколько бонусов получите)--------*/


    /*---------Удаление баллов за бездействие--------*/
    require_once(BONUS_COMPUTY_PLUGIN_DIR . '/index/cron.php');
    /*---------Удаление баллов за бездействие--------*/


/*---------очищение истории при удалении пользователя---------*/
function bfw_when_delete_user(int $user_id ) {
    global $wpdb;
    $table_name = $wpdb->prefix . 'bfw_history_computy';
    $wpdb->query($wpdb->prepare("DELETE FROM $table_name WHERE user = %d", $user_id)
    );

 }
add_action( 'delete_user', 'bfw_when_delete_user' );
/*---------очищение истории при удалении пользвоателя---------*/


/*Удаление временных баллов при очистке корзины*/
function action_woocommerce_before_cart_item_quantity_zero() {
    (new BfwPoints)->updateFastPoints(get_current_user_id(),0);
}
add_action( 'woocommerce_remove_cart_item', 'action_woocommerce_before_cart_item_quantity_zero', 10, 1 );

/*Действие сработает при изменении количества товаров*/
add_action( 'woocommerce_cart_item_set_quantity', 'action_function_name_4944', 10, 3 );
function action_function_name_4944( $cart_item_key, $quantity, $that ){
    $val = get_option( 'bonus_option_name' );
    $qty_cart= isset( $val['clear-fast-bonus-were-qty-cart'] ) ? intval( $val['clear-fast-bonus-were-qty-cart'] ) : 0;

    if($qty_cart){
        (new BfwPoints)->updateFastPoints(get_current_user_id(),0);
    }

}

/*Вывод списанных баллов в редактировании заказа админом*/
add_action( 'woocommerce_admin_order_totals_after_tax', 'bfw_inadminzakaz' );
function bfw_inadminzakaz( $order_id ){
    $order = wc_get_order($order_id);
    foreach( $order->get_items('fee') as $item_id => $item_fee ){
        $val = get_option( 'bonus_option_name' );

        if( $item_fee->get_name()== $val['bonus-points-on-cart']){
            $fee_total = $item_fee->get_total();
        }
    }
    if(isset($fee_total)){
        echo ' <tr><td class="label" >'.$val['bonus-points-on-cart'].':</td><td width="1%"></td>
		<td class="total">
		 <span class="woocommerce-Price-amount amount"><bdi>'.$fee_total.' <span class="woocommerce-Price-currencySymbol">'.  get_woocommerce_currency_symbol().'</span></bdi></span>				</td>
			</tr>';
    }
}
/*Вывод списанных баллов в редактировании заказа админом*/


/**
 * возможность менеджерам настраивать плагин
 * @version 3.2.0
 **/
$shop_manager = get_role( 'shop_manager' );
$shop_manager->add_cap( 'manage_options' );
$shop_manager->add_cap( 'edit_users' );
$shop_manager->add_cap( 'edit_user' );
function bonus_for_woo_manager_role_edit_capabilities( $roles ) {
    global $wpdb;
    $table_bfw = $wpdb->get_results("SELECT *,summa_start FROM " . $wpdb->prefix . "bfw_computy  ORDER BY summa_start + 0 asc");
    foreach ($table_bfw as $bfw) {
        $roles[] =   $bfw->slug;
    }
    $roles[] = 'subscriber';
    return $roles;
}
add_filter( 'woocommerce_shop_manager_editable_roles', 'bonus_for_woo_manager_role_edit_capabilities' );
/*возможность менеджерам настраивать плагин*/